/*
 *
 *
 */
#ifndef _LIB_UNI1P_
#define _LIB_UNI1P_

#include <windows.h>

#define EXPORT_HEAD 	__stdcall
#define EXPORT 	        _export __stdcall

//#define EXPORT_HEAD __declspec(dllexport)
typedef void* UNI1P_HANDLE;


#ifdef __cplusplus
extern "C" {
#endif

#if 0 // Emacsoid
}
#endif



/*****************************************************************
 * BASIC CONSTANTS                                               *
 *****************************************************************/
// Uni1P board VID and DID
#define UNI1P_VID               0x10b5
#define UNI1P_DID               0x9054

/*****************************************************************
 * RETURNS CODES OF FUNCTIONS WHICH RETURNS TYPE int             *
 *****************************************************************/
#define UNI1P_SUCCESS 				0
#define UNI1P_ERROR 				(-1)
#define UNI1P_ERROR_WD_INIT 		(-2)
#define UNI1P_ERROR_DMA_OPEN 		(-100)
#define UNI1P_ERROR_DMA_FINISH 		(-101)
#define UNI1P_ERROR_SYS_NOT_READY 	(-200)

/*****************************************************************
 * ACCESS FUNCTIONS                                              *
 *****************************************************************/
// access mode
typedef enum {
        BYTE_MODE = 1,
        DWORD_MODE = 4
} tAccessMode;

/*
 * Initialize library.
 */
int EXPORT_HEAD Uni1pInit(void);

/*
 * Open Uni1P board in 'slot'.
 */
UNI1P_HANDLE EXPORT_HEAD Uni1pOpenBoard(int slot);

/*
 * Close Uni1P board.
 */
void EXPORT_HEAD Uni1pCloseBoard(UNI1P_HANDLE hPlx);

/*
 * Reset PLX chip on the Uni1P board.
 */
void EXPORT_HEAD Uni1pResetBoard(UNI1P_HANDLE hPlx);

/*
 * Single accesses for read/write BYTE/DWORD from/to 'addr'
 */
void  EXPORT_HEAD Uni1pWriteDWord(UNI1P_HANDLE hPlx, DWORD addr, DWORD data);
DWORD EXPORT_HEAD Uni1pReadDWord(UNI1P_HANDLE hPlx, DWORD addr);
void  EXPORT_HEAD Uni1pWriteByte(UNI1P_HANDLE hPlx, DWORD addr, BYTE data);
BYTE  EXPORT_HEAD Uni1pReadByte(UNI1P_HANDLE hPlx, DWORD addr);

/*
 * Single accesses for read/write DWORD from/to PLX register on 'addr'
 */
void  EXPORT_HEAD Uni1pWriteReg(UNI1P_HANDLE hPlx, DWORD addr, DWORD data);
DWORD EXPORT_HEAD Uni1pReadReg(UNI1P_HANDLE hPlx, DWORD addr);

/*
 * Write/read block of data 'buf' with length 'size' (in bytes) to/from 'addr' in
 * address fix/autoincrement mode.
 */
int EXPORT_HEAD Uni1pBlock( UNI1P_HANDLE hPlx, BOOL fIsRead, DWORD addr,
                BYTE *buf, DWORD size, tAccessMode mode, BOOL fix_addr);

/*
 * DMA write/read block of data 'buf' with length 'size' (in bytes) to/from 'addr' in
 * address fix/autoincrement mode.
 */
int EXPORT_HEAD Uni1pDMA(   UNI1P_HANDLE hPlx, BOOL fIsRead, DWORD addr,
                BYTE *buf, DWORD size, tAccessMode mode, BOOL fix_addr);


/*****************************************************************
 * FPGA CONFIGURATION                                            *
 *****************************************************************/

// FPGA chip select
typedef enum {
        UNI1P_FPGA_SYSTEM  = 256,
        UNI1P_FPGA_MASTER  = 1,
        UNI1P_FPGA_COMM    = 2,
        UNI1P_FPGA_DX64_M0 = 4,
        UNI1P_FPGA_DX64_M1 = 8,
        UNI1P_FPGA_DX64_M2 = 16,
        UNI1P_FPGA_DX64_M3 = 32
} tUni1pFpgaChip;

// status
typedef enum {
		UNI1P_FPGA_SYSTEM_CLEAR	= 0x00000100,
		UNI1P_FPGA_SYSTEM_RESET	= 0x00000200,
		UNI1P_FPGA_SYSTEM_LED 	= 0x00000400,
		UNI1P_FPGA_SYSTEM_DONE 	= 0x00001000,
		UNI1P_FPGA_SYSTEM_PROG 	= 0x00002000,
		UNI1P_FPGA_OTHERS_RESET	= 0x04000000,
		UNI1P_FPGA_OTHERS_READY	= 0x08000000,
		UNI1P_FPGA_OTHERS_DONE 	= 0x10000000,
		UNI1P_FPGA_OTHERS_PROG 	= 0x20000000
} tUni1pFpgaStatus;

/*
 * initialize library
 */
int EXPORT_HEAD Uni1pFpgaInit( UNI1P_HANDLE hPlx );

/*
 * Returns status.
 */
tUni1pFpgaStatus EXPORT_HEAD Uni1pFpgaGetStat( UNI1P_HANDLE hPlx );

/*
 * Reset Fpga
 */
int EXPORT_HEAD Uni1pFpgaReset(     UNI1P_HANDLE hPlx,
                        			tUni1pFpgaChip fpgas );
/*
 * Unreset Fpga
 */
int EXPORT_HEAD Uni1pFpgaUnreset(   UNI1P_HANDLE hPlx,
                        			tUni1pFpgaChip fpgas );

/*
 * Clean fpgas configuration.
 * Function begins with clear all application FPGAs.
 * Then clear sytem FPGA only if selected by 'fpga' parameter.
 */
int EXPORT_HEAD Uni1pFpgaClean(     UNI1P_HANDLE hPlx,
                        			tUni1pFpgaChip fpga );


/*
 * Load configuration with byte count 'size'
 * from buffer 'buff' to destination 'fpga'
 */
int EXPORT_HEAD Uni1pFpgaLoad(      UNI1P_HANDLE hPlx,
									tUni1pFpgaChip fpga,
									int size,
									BYTE *buff );


/*****************************************************************
 * HOST PORT ACCESS                                              *
 *****************************************************************/

// DSP module select
typedef enum {
        UNI1P_DSP_M0 = 1,
        UNI1P_DSP_M1 = 2,
        UNI1P_DSP_M2 = 4,
        UNI1P_DSP_M3 = 8
} tUni1pHpiModule;

// HPI status flags
typedef enum {
		UNI1P_HPI_READY = 0x08000000
} tUni1pHpiStatus;


/*
 * initialize library and HPI
 * on success returns 0, else -1
 */
int EXPORT_HEAD Uni1pHpiInit( UNI1P_HANDLE hPlx );

/*
 * Reset DSP
 */
int EXPORT_HEAD Uni1pHpiDspReset(   UNI1P_HANDLE hPlx,
                        tUni1pHpiModule dsp );

/*
 * Startup DSP
 */
int EXPORT_HEAD Uni1pHpiDspRun(     UNI1P_HANDLE hPlx,
                        tUni1pHpiModule dsp );

/*
 * STANDARD DRIVER
 * Function returns status of HPI driver
 */
tUni1pHpiStatus EXPORT_HEAD Uni1pHpiGetStat( UNI1P_HANDLE hPlx );

/*
 * STANDARD DRIVER
 * Write/read data block of length 'size' (in DWORDs)
 * from/to buffer 'buff' to/from DSP address 'addr'
 */
int EXPORT_HEAD Uni1pHpiBlock( UNI1P_HANDLE hPlx,
                        BOOL read,
                        tUni1pHpiModule dsp,
                        DWORD addr,
                        int size,
                        DWORD *buff );


/*
 * STANDARD DRIVER
 * Write/read data block of length 'size' (in DWORDs)
 * from/to buffer 'buff' to/from DSP address 'addr' using DMA
 */
int EXPORT_HEAD Uni1pHpiDma( UNI1P_HANDLE hPlx,
                        BOOL read,
                        tUni1pHpiModule dsp,
                        DWORD addr,
                        int size,
                        DWORD *buff );

/*
 * HIGH PRIORITY DRIVER
 * Function returns status of HPI driver
 */
tUni1pHpiStatus EXPORT_HEAD Uni1pHpiHiGetStat( UNI1P_HANDLE hPlx );


/*
 * HIGH PRIORITY DRIVER
 * Write/read data block of length 'size' (in DWORDs)
 * from/to buffer 'buff' to/from DSP address 'addr'
 */
int EXPORT_HEAD Uni1pHpiHiBlock( UNI1P_HANDLE hPlx,
                        BOOL read,
                        tUni1pHpiModule dsp,
                        DWORD addr,
                        int size,
                        DWORD *buff );


/*
 * HIGH PRIORITY DRIVER
 * Write single dword to DSP address 'dest'
 */
void EXPORT_HEAD Uni1pHpiHiWriteDWord( UNI1P_HANDLE hPlx,
                        tUni1pHpiModule dsp,
                        DWORD dest,
                        DWORD data);

/*
 * HIGH PRIORITY DRIVER
 * Read single dword from DSP address 'src'.
 */
DWORD EXPORT_HEAD Uni1pHpiHiReadDWord(  UNI1P_HANDLE hPlx,
                        tUni1pHpiModule dsp,
                        DWORD src);

#ifdef __cplusplus
}
#endif

#endif

